from __future__ import absolute_import
from nlpaug.augmenter.word.word_augmenter import *
from nlpaug.augmenter.word.wordnet import *
from nlpaug.augmenter.word.random import *
from nlpaug.augmenter.word.word_embs import *
from nlpaug.augmenter.word.word_embs_aug import *
from nlpaug.augmenter.word.glove import *
from nlpaug.augmenter.word.word2vec import *
from nlpaug.augmenter.word.fasttext import *
from nlpaug.augmenter.word.bert import *
from nlpaug.augmenter.word.tfidf import *
from nlpaug.augmenter.word.spelling import *
from nlpaug.augmenter.word.stopwords import *