from __future__ import absolute_import
from nlpaug.augmenter.audio.audio_augmenter import *
from nlpaug.augmenter.audio.noise import *
from nlpaug.augmenter.audio.shift import *
from nlpaug.augmenter.audio.speed import *
from nlpaug.augmenter.audio.pitch import *
from nlpaug.augmenter.audio.loudness import *
from nlpaug.augmenter.audio.crop import *
from nlpaug.augmenter.audio.mask import *
